#!/bin/sh

################################################################################
#    backupDAT 1.2.                                                            #
#    Copyright (C) 2003 Luca Bertoncello                                       #
#    Waltherstrae, 11D - 01067 Dresden Deutschland                            #
#    E-Mail: lucabert@lucabert.com                                             #
#                                                                              #
#    This program is free software; you can redistribute it and/or modify      #
#    it under the terms of the GNU General Public License as published by      #
#    the Free Software Foundation; version 2 of the License.                   #
#                                                                              #
#    This program is distributed in the hope that it will be useful,           #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of            #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
#    GNU General Public License for more details.                              #
#                                                                              #
#    You should have received a copy of the GNU General Public License         #
#    along with this program; if not, write to the Free Software               #
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA #
################################################################################

# Default values (you can change them by parameter):
# Device to write to
TAPEDEV=/dev/nst0
# Set AUTOEJECT to 1 in order to eject tape at the end of backup process
# Set AUTOEJECT to 0 in order to rewind tape at the end of backup process, but not eject it
AUTOEJECT=0
# Set VERIFYDATA to 1 in order to verify saved data
VERIFYDATA=1

# Configurazion strings. You don't need to modify these
MAIN_DIR=tmpBackup
BACKUP_FILE_PREFIX=bk
BACKUP_FILE_SUFFIX=.tgz
TAROPTS=cfps
FILE_BACKUP_DIR=

# Path of the commands. You can change them, with the path in your system.
CMDAWK=/bin/awk
CMDCAT=/bin/cat
CMDCOMPRESS=/bin/gzip
CMDDATE=/bin/date
CMDDU=/usr/bin/du
CMDECHO=/bin/echo
CMDEXPR=/usr/bin/expr
CMDGREP=/bin/grep
CMDMAIL=/bin/mail
CMDMKDIR=/bin/mkdir
CMDMT=/bin/mt
CMDPERL=/usr/bin/perl
CMDRM=/bin/rm
CMDSLEEP=/bin/sleep
CMDTAR=/bin/tar

# Please DO NOT modify under this line!!
filesList="$CMDAWK $CMDCAT $CMDCOMPRESS $CMDDATE $CMDDU $CMDECHO $CMDEXPR $CMDGREP $CMDMAIL $CMDMKDIR $CMDMT $CMDPERL $CMDRM $CMDSLEEP $CMDTAR"

# Path of Backup/Restore script. DO NOT ALTER IT!!
RESTOREFILE=$0

# Check all files
function checkFiles
{
  FILESOK=0
  for fName in $filesList; do
    if test ! -e $fName; then
      $CMDECHO "'$fName': file not exist!"
      FILESOK=1
    fi
  done

  return $FILESOK
}

# Creates a compressed TAR for given directory
function compressDir
{
  $CMDTAR $TAROPTS - ${2} 2> /dev/null | $CMDCOMPRESS - > ${1} 2> /dev/null

  return $?
}

# Searches for temp directory
function searchTmpDir
{
  tmpDirList="/var/tmp /var/temp /usr/tmp /usr/temp /tmp"

  for dir in $tmpDirList; do
    if test -d $dir; then
      FILE_BACKUP_DIR=$dir
    fi
  done
}

# Checks for Tape
function checkTape
{
  $CMDMT -f $TAPEDEV stat | $CMDGREP ONLINE > /dev/null 2> /dev/null
  if test $? -ne 0; then
    $CMDECHO "ERROR!! Tape not found!"
    $CMDECHO "Please enter tape into DAT and start the program again"
    exit
  fi
}

# Exit from program, rewinding or ejecting tape (see configurazion/parameter)
function endOfProcedure
{
  $CMDSLEEP 1
  if test $AUTOEJECT -eq 1; then
    $CMDECHO "==> Ejecting tape..."
    $CMDMT -f $TAPEDEV offline
  else
    $CMDECHO "==> Rewinding tape..."
    $CMDMT -f $TAPEDEV rewind
  fi
}

# Checks directories to be backupped
function checkDirsFile
{
  FILEINVALID=0
  if test "$*"; then
    for dir in $*; do
      if test ! -d $dir; then
        FILEINVALID=1
        $CMDECHO "     '$dir': invalid directory!!"
      fi
      $CMDECHO "$dir" | $CMDAWK '{if($1 !~ /\/$/) exit 0; else exit 1;}'
      if test $? -ne 0; then
        FILEINVALID=1
        $CMDECHO "     '$dir': you don't have to add '/' at the end!!"
      fi
    done
  else
    FILEINVALID=1
  fi

  return $FILEINVALID
}

function backup
{
  if test ${1} && test -f ${1}; then
    DIR_BACKUPPED=`$CMDCAT ${1} | $CMDAWK '{ if(($0) && ($0 !~ /#/)) print $0; }'`
  else
    if test ${1}; then
      $CMDECHO "Invalid backup file!"
    fi
    FILEOK=0
    while test $FILEOK -eq 0
    do
      $CMDECHO -n "Enter file with list of directories to backup: "
      read CONFILE
      if test ! $CONFILE || test ! -f $CONFILE; then
        $CMDECHO "Invalid backup file!"
      else
        FILEOK=1
      fi
    done
    DIR_BACKUPPED=`$CMDCAT $CONFILE | $CMDAWK '{ if(($0) && ($0 !~ /#/)) print $0; }'`
  fi
  $CMDECHO "==> Parsing config file..."
  checkDirsFile $DIR_BACKUPPED
  if test $? -ne 0; then
    $CMDECHO "In order to start backup procedure, you MUST";
    $CMDECHO "create a file with a list of directories, one per line,";
    $CMDECHO "WITHOUT / at the end.";
    $CMDECHO "Program stopped.";
    exit
  fi
  checkTape

  # Calculate length of directories
  LEN=21
  for DIR in $DIR_BACKUPPED; do
    LEN=`$CMDPERL -e '$dir = $ARGV[0]; $len = $ARGV[1]; $prefix = $ARGV[2]; if((length($dir) + length($prefix)) > $len) { $len = length($dir) + length($prefix); } print $len;' $DIR $LEN $BACKUP_FILE_PREFIX`;
  done

  ERROR=0
  # Creates the Backup
  CURDATE=`$CMDDATE +%d_%m_%Y`;
  BACKUP_DIR=$FILE_BACKUP_DIR/$MAIN_DIR/backup$CURDATE
  $CMDECHO "==> Creating backup files..."
  $CMDMKDIR $FILE_BACKUP_DIR/$MAIN_DIR
  $CMDMKDIR $BACKUP_DIR
  for file in $DIR_BACKUPPED; do
      nfile=`$CMDECHO $file | sed -e "s/\//\_/g"`
      file=$file/
      fileBackupName=$BACKUP_FILE_PREFIX$nfile$BACKUP_FILE_SUFFIX
      pathFileBackup=$BACKUP_DIR/$fileBackupName
      $CMDPERL -e '$file = $ARGV[0]; $len = $ARGV[1]; $len += 25; $format = "%-$len"."s"; printf($format, "     Compressing '$file'...");' $file $LEN
      compressDir $pathFileBackup $file
      if test $? -eq 0; then
        $CMDECHO "OK"
      else
        $CMDECHO "FAILED!"
        ERROR=1
      fi
  done
  if test $ERROR -ne 0; then
    $CMDECHO "ERROR in compressing files!";
    $CMDECHO "Program stopped.";
    exit
  fi

  $CMDECHO "==> Rewinding tape..."
  $CMDMT -f $TAPEDEV rewind

  # Creates index file
  CURDATE=`$CMDDATE +%d/%m/%Y`;
  $CMDECHO "==> Creating index on tape..."
  TMPINDEXFILE=$BACKUP_DIR/index_`$CMDDATE +%d%m%Y_%H%M%S`;
  $CMDECHO "Backup date: $CURDATE" > $TMPINDEXFILE
  NDIR=0
  $CMDPERL -e '$len = $ARGV[0]; $prefix = $ARGV[1]; $suffix = $ARGV[2]; $len1 = $len + 5; $len += length($prefix) + length($suffix) - 2; $format = "| Track | %-$len"."s | %-$len1"."s | %-8s |\n"; printf($format, "File", "Remarks", "Length");' $LEN $BACKUP_FILE_PREFIX $BACKUP_FILE_SUFFIX >> $TMPINDEXFILE
  $CMDPERL -e '$len = $ARGV[0]; $prefix = $ARGV[1]; $suffix = $ARGV[2]; print "+-------+"; for($l = 0; $l < ($len + length($prefix) + length($suffix)); $l++) {print "-";} print "+"; for($l = 0; $l < ($len + 7); $l++) {print "-";} print "+----------+\n";' $LEN $BACKUP_FILE_PREFIX $BACKUP_FILE_SUFFIX >> $TMPINDEXFILE
  $CMDPERL -e '$len = $ARGV[0]; $nDir = $ARGV[1]; $prefix = $ARGV[2]; $suffix = $ARGV[3]; $len1 = $len + 5; $len += length($prefix) + length($suffix) - 2; $format = "| %5d | %-$len"."s | %-$len1"."s | %-8s |\n"; printf($format, $nDir, "index.txt", "This file", "");' $LEN $NDIR $BACKUP_FILE_PREFIX $BACKUP_FILE_SUFFIX >> $TMPINDEXFILE
  NDIR=`$CMDEXPR $NDIR + 1`
  FILELEN=`$CMDDU -ksh backupDAT.sh | $CMDAWK '{ print $1."b"; }'`
  $CMDPERL -e '$len = $ARGV[0]; $nDir = $ARGV[1]; $prefix = $ARGV[2]; $suffix = $ARGV[3]; $fileLen = $ARGV[4]; $len1 = $len + 5; $len += length($prefix) + length($suffix) - 2; $format = "| %5d | %-$len"."s | %-$len1"."s | %8s |\n"; printf($format, $nDir, "backupDAT.sh", "Backup/Restore script", $fileLen);' $LEN $NDIR $BACKUP_FILE_PREFIX $BACKUP_FILE_SUFFIX $FILELEN >> $TMPINDEXFILE
  NDIR=`$CMDEXPR $NDIR + 1`
  for DIR in $DIR_BACKUPPED; do
    file=$DIR
    nfile=`$CMDECHO $file | sed -e "s/\//\_/g"`
    fileBackupName=$BACKUP_FILE_PREFIX$nfile$BACKUP_FILE_SUFFIX
    pathFileBackup=$BACKUP_DIR/$fileBackupName
    FILELEN=`$CMDDU -ksh $pathFileBackup | $CMDAWK '{ print $1."b"; }'`
    $CMDPERL -e '$dir = $ARGV[0]; $len = $ARGV[1]; $nDir = $ARGV[2]; $prefix = $ARGV[3]; $suffix = $ARGV[4]; $fileLen = $ARGV[5]; $len1 = $len + 5; $len += length($prefix) + length($suffix) - 2; $file = "$prefix$dir$suffix"; $file =~ s/\//\_/g; $format = "| %5d | %-$len"."s | %-$len1"."s | %8s |\n"; printf($format, $nDir, $file, "Backup $dir", $fileLen);' $DIR $LEN $NDIR $BACKUP_FILE_PREFIX $BACKUP_FILE_SUFFIX $FILELEN >> $TMPINDEXFILE;
    NDIR=`$CMDEXPR $NDIR + 1`
  done
  $CMDPERL -e '$len = $ARGV[0]; $prefix = $ARGV[1]; $suffix = $ARGV[2]; print "+-------+"; for($l = 0; $l < ($len + length($prefix) + length($suffix)); $l++) {print "-";} print "+"; for($l = 0; $l < ($len + 7); $l++) {print "-";} print "+----------+\n";' $LEN $BACKUP_FILE_PREFIX $BACKUP_FILE_SUFFIX >> $TMPINDEXFILE
  TOTALSPACE=`$CMDDU -ksh $BACKUP_DIR | $CMDAWK '{ print $1."b"; }'`
  $CMDECHO "Total: $TOTALSPACE" >> $TMPINDEXFILE
  $CMDTAR cf $TAPEDEV $TMPINDEXFILE 2> /dev/null

  $CMDECHO "==> Storing backup/restore script on tape..."
  $CMDTAR cf $TAPEDEV $RESTOREFILE 2> /dev/null

  # Storing the Backup
  $CMDECHO "==> Storing backup on tape..."
  for file in $DIR_BACKUPPED; do
      nfile=`$CMDECHO $file | sed -e "s/\//\_/g"`
      file=$file/
      fileBackupName=$BACKUP_FILE_PREFIX$nfile$BACKUP_FILE_SUFFIX
      pathFileBackup=$BACKUP_DIR/$fileBackupName
      $CMDPERL -e '$file = $ARGV[0]; $len = $ARGV[1]; $len += 25; $format = "%-$len"."s"; printf($format, "     Storing '$file' on tape...");' $file $LEN
      $CMDTAR cf $TAPEDEV $pathFileBackup 2> /dev/null
      if test $? -eq 0; then
        $CMDECHO "OK"
      else
        $CMDECHO "FAILED!"
        ERROR=1
      fi
  done
  if test $ERROR -ne 0; then
    $CMDECHO "ERROR in storing files!";
    $CMDECHO "Program stopped.";
    exit
  fi

  VERIFY=0
  if test $VERIFYDATA -eq 1; then
    $CMDECHO "==> Rewinding tape..."
    $CMDMT -f $TAPEDEV rewind
    $CMDSLEEP 1

    # Verifying the Backup
    $CMDECHO "==> Verifying backup..."
    $CMDMT -f $TAPEDEV fsf 2    # Jump over index and restore script
    $CMDSLEEP 1
    for file in $DIR_BACKUPPED; do
        nfile=`$CMDECHO $file | sed -e "s/\//\_/g"`
        file=$file/
        fileBackupName=$BACKUP_FILE_PREFIX$nfile$BACKUP_FILE_SUFFIX
        pathFileBackup=$BACKUP_DIR/$fileBackupName
        $CMDPERL -e '$file = $ARGV[0]; $len = $ARGV[1]; $len += 25; $format = "%-$len"."s"; printf($format, "     Verifying '$file'...");' $file $LEN
        $CMDTAR tf $TAPEDEV > /dev/null 2> /dev/null
        if test $? -eq 0; then
          $CMDECHO "OK"
        else
          $CMDECHO "FAILED!"
          VERIFY=1
        fi
        $CMDSLEEP 1
        $CMDMT -f $TAPEDEV fsf 1
    done
  fi

  $CMDECHO "==> Removing temp files..."
  $CMDRM -Rf $FILE_BACKUP_DIR/$MAIN_DIR

  endOfProcedure

  if test $VERIFY -eq 0; then
    $CMDECHO "==> Backup completed successfully!"
    $CMDECHO "Saved $TOTALSPACE in `$CMDEXPR $NDIR - 2` file(s)"
  else
    $CMDECHO "==> ERRORS IN BACKUP!!"
  fi
}

function displayCommandPrompt
{
  $CMDECHO ""
  $CMDECHO "Available functions:"
  $CMDECHO " - extract <nFile|fileName>      extract file with number <nFile>"
  $CMDECHO "                                 or with name <fileName>."
  $CMDECHO "                                 (? and * are allowed)."
  $CMDECHO " - extract all                   extract all files on tape."
  $CMDECHO " - list                          display list of files on tape."
  $CMDECHO " - quit                          exit."
}

function displayList
{
  $CMDECHO ""
  $CMDCAT ${1}
  displayCommandPrompt
}

function extract
{
  PATHOK=0
  while test $PATHOK -eq 0
  do
    $CMDECHO -n "Enter path to extract file(s) to: "
    read PATHTOCOPY
    if test ! $PATHTOCOPY || test ! -d $PATHTOCOPY; then
      $CMDECHO "Invalid path!"
    else
      PATHOK=1
    fi
  done

  # Calculate length of directories
  LEN=0
  for FILE in $FILENAMES; do
    LEN=`$CMDPERL -e '$file = $ARGV[0]; $len = $ARGV[1]; if(length($file) > $len) { $len = length($file); } print $len;' $FILE $LEN`;
  done

  ERRORS=0
  $CMDECHO "==> Positioning tape..."
  # Jump to first file in the list
  firstFile=`$CMDECHO "$*" | $CMDAWK '{ print $1; }'`
  $CMDMT -f $TAPEDEV asf $firstFile
  POS=$firstFile
  $CMDECHO "==> Extracting file(s)..."
  for nr in $*; do
    JMP=`$CMDEXPR $nr - $POS`;
    if test $JMP -gt 0; then
      $CMDMT -f $TAPEDEV fsf $JMP;
      POS=$nr;
    fi
    nFile=0
    for f in $FILENAMES; do
      if test $nFile -eq `$CMDEXPR $nr - 2`; then
        file=$f;
      fi
      nFile=`$CMDEXPR $nFile + 1`
    done

    $CMDPERL -e '$file = $ARGV[0]; $len = $ARGV[1]; $len += 25; $format = "%-$len"."s"; printf($format, "     Extracting '$file'...");' $file $LEN
    $CMDTAR xOf $TAPEDEV > $PATHTOCOPY/$file 2> /dev/null
    if test $? -eq 0; then
      $CMDECHO "OK"
    else
      $CMDECHO "FAILED!"
      ERRORS=1
    fi
    $CMDMT -f $TAPEDEV fsf 1;
    POS=`$CMDEXPR $POS + 1`
  done

  if test $ERRORS -eq 0; then
    $CMDECHO "==> Extract completed successfully!"
  else
    $CMDECHO "==> ERRORS IN EXTRATING FILE(S)!!"
  fi
}

function restore
{
  checkTape

  $CMDECHO "==> Rewinding tape..."
  $CMDMT -f $TAPEDEV rewind

  $CMDECHO "==> Reading index from tape..."
  $CMDMKDIR $FILE_BACKUP_DIR/$MAIN_DIR
  TMPINDEXFILE=$FILE_BACKUP_DIR/$MAIN_DIR/index_`$CMDDATE +%d%m%Y_%H%M%S`;
  $CMDTAR xfO $TAPEDEV > $TMPINDEXFILE
  FILELIST=`$CMDCAT $TMPINDEXFILE | $CMDAWK '{ if(($0 ~ /^\|/) && ($2 !~ /Track/) && ($2 > 1)) print $2; }'`
  FILENAMES=`$CMDCAT $TMPINDEXFILE | $CMDAWK '{ if(($0 ~ /^\|/) && ($2 !~ /Track/) && ($2 > 1)) print $4; }'`

  displayList $TMPINDEXFILE

  QUIT=0

  while test $QUIT -eq 0
  do
    MATCH=0
    $CMDECHO -n "=> "
    read COMMAND
    if test "$COMMAND"; then
      case "$COMMAND" in
        list)
          displayList $TMPINDEXFILE;
          MATCH=1;
          ;;
        quit)
          QUIT=1;
          MATCH=1;
          ;;
      esac
      if test "`$CMDECHO $COMMAND | $CMDAWK '{print substr($0, 1, 8)}'`" = "extract "; then
        TRACKSTOEXTRACT=`$CMDECHO $COMMAND | $CMDAWK '{print substr($0, 9)}'`;
        MATCH=1;
        DONEEXTR=0
        if test "$TRACKSTOEXTRACT" = "0" || test "$TRACKSTOEXTRACT" = "1"; then
          $CMDECHO "Tracks 0 and 1 are reserved (index file and restore script)!"
          $CMDECHO "You don't need to extract them."
          DONEEXTR=1
        fi
        if test "$TRACKSTOEXTRACT" = "all" || test "$TRACKSTOEXTRACT" = "*"; then
          extract $FILELIST;
          displayCommandPrompt;
          DONEEXTR=1
        fi
        for nr in $FILELIST; do
          if test "$TRACKSTOEXTRACT" = $nr; then
            extract $TRACKSTOEXTRACT;
            displayCommandPrompt;
            DONEEXTR=1
          fi
        done
        if test $DONEEXTR -eq 0; then
          nFile=2
          FILETOEXTRACTLIST=""
          for f in $FILENAMES; do
            $CMDPERL -e '$str = $ARGV[0]; $pat = $ARGV[1]; $pat =~ s/\?/\(\[\\w\\d\\\.\]\)/g; $pat =~ s/\*/\(\[\\w\\d\\\.\]\)\*/g; if($str =~ /^$pat$/) {exit 0;} else {exit 1;};' "$f" "$TRACKSTOEXTRACT"
            if test $? -eq 0; then
              if test "$FILETOEXTRACTLIST" != ""; then
                FILETOEXTRACTLIST="$FILETOEXTRACTLIST $nFile";
              else
                FILETOEXTRACTLIST=$nFile;
              fi
            fi
            nFile=`$CMDEXPR $nFile + 1`
          done
          if test "$FILETOEXTRACTLIST" != ""; then
            extract $FILETOEXTRACTLIST;
            displayCommandPrompt;
            DONEEXTR=1
          fi
        fi
        if test $DONEEXTR -eq 0; then
          $CMDECHO "Invalid file/track!"
        fi
      fi
      if test $MATCH -eq 0; then
        $CMDECHO "Unknown command!"
      fi
    fi
  done

  $CMDRM -Rf $FILE_BACKUP_DIR/$MAIN_DIR

  endOfProcedure
}

function help
{
  if test "$AUTOEJECT" = "1"; then
    AUTOEJECTSTR="true";
  else
    AUTOEJECTSTR="false";
  fi
  if test "$VERIFYDATA" = "1"; then
    VERIFYDATASTR="true";
  else
    VERIFYDATASTR="false";
  fi
  $CMDECHO "backupDAT 1.2."
  $CMDECHO "Script for backup and restore on tape."
  $CMDECHO "By Luca Bertoncello 2003. GPL v.2.0 Licence"
  $CMDECHO "For information about the program and the author:"
  $CMDECHO "               http://www.lucabert.de/"
  $CMDECHO ""
  $CMDECHO "Parameters:"
  $CMDECHO "  help                  Display this help"
  $CMDECHO "  backup                Run backup procedure (default)"
  $CMDECHO "  restore               Run restore procedure"
  $CMDECHO "  bklist=<file>         Use <file> for backup list."
  $CMDECHO "                        If not specified, it will be asked."
  $CMDECHO "  dev=<dev>             Use <dev> as device to write to"
  $CMDECHO "                        (default: $TAPEDEV)."
  $CMDECHO "  eject=<true|false>    Eject tape at the end of backup/restore"
  $CMDECHO "                        process (default: $AUTOEJECTSTR)."
  $CMDECHO "  verify=<true|false>   Verify saved files at the end of backup"
  $CMDECHO "                        process (default: $VERIFYDATASTR)."
  $CMDECHO "  tmpdir=<dir>          Use <dir> as temporary directory."
  $CMDECHO "                        (Default: $FILE_BACKUP_DIR)."
  $CMDECHO "  email=<address>       Send the result of backup to the"
  $CMDECHO "                        given E-Mail address."
  $CMDECHO ""
  $CMDECHO "Starting the program without parameters cause starting backup"
}

# Check if all files are installed
checkFiles
if test $? -ne 0; then
  $CMDECHO "Program stopped.";
  exit
fi

# Search temporary directory
searchTmpDir

EMAILTO=""
# Parsing arguments
BACKUPRESTORE=0
for ARG in $*; do
  RECOGNIZED=0
  if test "$ARG" = "backup"; then
    BACKUPRESTORE=0;
    RECOGNIZED=1
  fi
  if test "$ARG" = "restore"; then
    BACKUPRESTORE=1;
    RECOGNIZED=1
  fi
  if test "$ARG" = "help"; then
    help;
    RECOGNIZED=1;
    exit
  fi
  if test "`$CMDECHO $ARG | $CMDAWK '{print substr($0, 1, 6)}'`" = "email="; then
    EMAILTO=`$CMDECHO $ARG | $CMDAWK '{print substr($0, 7)}'`;
    RECOGNIZED=1
  fi
  if test "`$CMDECHO $ARG | $CMDAWK '{print substr($0, 1, 7)}'`" = "tmpdir="; then
    FILE_BACKUP_DIR=`$CMDECHO $ARG | $CMDAWK '{print substr($0, 8)}'`;
    RECOGNIZED=1
  fi
  if test "`$CMDECHO $ARG | $CMDAWK '{print substr($0, 1, 7)}'`" = "bklist="; then
    BKLIST=`$CMDECHO $ARG | $CMDAWK '{print substr($0, 8)}'`;
    RECOGNIZED=1
  fi
  if test "`$CMDECHO $ARG | $CMDAWK '{print substr($0, 1, 4)}'`" = "dev="; then
    TAPEDEV=`$CMDECHO $ARG | $CMDAWK '{print substr($0, 5)}'`;
    RECOGNIZED=1
  fi
  if test "`$CMDECHO $ARG | $CMDAWK '{print substr($0, 1, 6)}'`" = "eject="; then
    STRAUTOEJECT=`$CMDECHO $ARG | $CMDAWK '{print substr($0, 7)}'`;
    RECOGNIZED=1;
    if test "$STRAUTOEJECT" = "true" || test "$STRAUTOEJECT" = "1"; then
      AUTOEJECT=1
    elif test "$STRAUTOEJECT" = "false" || test "$STRAUTOEJECT" = "0"; then
      AUTOEJECT=0
    else
      $CMDECHO "Unknown option '$STRAUTOEJECT'. Using default!";
    fi
  fi
  if test "`$CMDECHO $ARG | $CMDAWK '{print substr($0, 1, 7)}'`" = "verify="; then
    STRVERIFY=`$CMDECHO $ARG | $CMDAWK '{print substr($0, 8)}'`;
    RECOGNIZED=1;
    if test "$STRVERIFY" = "true" || test "$STRVERIFY" = "1"; then
      VERIFYDATA=1
    elif test "$STRVERIFY" = "false" || test "$STRVERIFY" = "0"; then
      VERIFYDATA=0
    else
      $CMDECHO "Unknown option '$STRVERIFY'. Using default!";
    fi
  fi

  if test $RECOGNIZED -eq 0; then
    $CMDECHO "Unknown command '$ARG'";
    $CMDECHO "Try backupDAT.sh help for information.";
    exit
  fi
done

if test ! -d $FILE_BACKUP_DIR; then
  $CMDECHO "ERROR: '$FILE_BACKUP_DIR' - invalid directory!!";
  $CMDECHO "Program stopped.";
  exit
fi

if test $BACKUPRESTORE -eq 0; then
  if test "$EMAILTO" = ""; then
    backup $BKLIST
  else
    backup $BKLIST | $CMDMAIL -s "Backup result" $EMAILTO
  fi
else
  restore
fi
